@@ -149,7 +149,7 @@ validate_email_addresses (const char *key,
                           GVariant *options,
                           GError **error)
 {
-  const char *const *strings = g_variant_get_strv (value, NULL);
+  g_autofree const char *const *strings = g_variant_get_strv (value, NULL);
   int i;
 
   for (i = 0; strings[i]; i++)
@@ -219,7 +219,7 @@ handle_compose_email (XdpDbusEmail *object,
   REQUEST_AUTOLOCK (request);
 
   impl_request = xdp_dbus_impl_request_proxy_new_sync (g_dbus_proxy_get_connection (G_DBUS_PROXY (impl)),
-                                                       G_DBUS_PROXY_FLAGS_NONE,
+                                                       G_DBUS_PROXY_FLAGS_DO_NOT_LOAD_PROPERTIES,
                                                        g_dbus_proxy_get_name (G_DBUS_PROXY (impl)),
                                                        request->id,
                                                        NULL, &error);
