@@ -24,5 +24,5 @@
 #include <gio/gio.h>
 
 GDBusInterfaceSkeleton * open_uri_create (GDBusConnection *connection,
-                                          const char      *dbus_name);
-
+                                          const char      *dbus_name,
+                                          gpointer         lockdown);
